part of 'sort_index_bar.dart';

double screenWidth(BuildContext context) => MediaQuery.of(context).size.width;
double screenHeight(BuildContext context) => MediaQuery.of(context).size.height;

class IndexBar extends StatefulWidget {
  final void Function(String str) indexBarCallBack;

  const IndexBar({Key? key, required this.indexBarCallBack}) : super(key: key);

  @override
  _IndexBarState createState() => _IndexBarState();
}

class _IndexBarState extends State<IndexBar> {
  Color _backgroundColor = Color.fromRGBO(1, 1, 1, 0);
  Color _titleColor = Colors.grey;
  String _currentTitle = 'A';
  double _indocatorY = 0;
  bool _indocatorHidden = true;

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onVerticalDragDown: (DragDownDetails details) {
        double y = details.localPosition.dy;
        int index = getIndex(context, y);
        widget.indexBarCallBack(INDEX_WORDS[index]);
        setState(() {
          _backgroundColor = Color.fromRGBO(1, 1, 1, 0.5);
          _titleColor = Colors.white;
          _currentTitle = INDEX_WORDS[index];
          _indocatorY = 2.2 / INDEX_WORDS.length * index - 1.1;
          _indocatorHidden = false;
        });
      },
      onVerticalDragUpdate: (DragUpdateDetails details) {
        double y = details.localPosition.dy;
        int index = getIndex(context, y);
        widget.indexBarCallBack(INDEX_WORDS[index]);
        setState(() {
          _backgroundColor = Color.fromRGBO(1, 1, 1, 0.5);
          _titleColor = Colors.white;
          _currentTitle = INDEX_WORDS[index];
          _indocatorY = 2.2 / INDEX_WORDS.length * index - 1.1;
          _indocatorHidden = false;
        });
      },
      onVerticalDragCancel: () {
        setState(() {
          _backgroundColor = Color.fromRGBO(1, 1, 1, 0.0);
          _titleColor = Colors.grey;
          _indocatorHidden = true;
        });
      },
      onVerticalDragEnd: (DragEndDetails details) {
        setState(() {
          _backgroundColor = Color.fromRGBO(1, 1, 1, 0.0);
          _titleColor = Colors.grey;
          _indocatorHidden = true;
        });
      },
      onTapCancel: () {
        print('取消了');
        setState(() {
          _backgroundColor = Color.fromRGBO(1, 1, 1, 0.0);
          _titleColor = Colors.grey;
          _indocatorHidden = true;
        });
      },
      child: Container(
          height: screenHeight(context) / 2,
          width: 100,
          child: Row(
            children: <Widget>[
              Container(
                alignment: Alignment(0, _indocatorY),
                width: 70,
                child: _indocatorHidden
                    ? Container()
                    : Stack(
                        alignment: Alignment(-0.2, 0),
                        children: <Widget>[
                          Image.asset(
                            'images/气泡.png',
                            width: 60,
                            package: 'sort_index_bar',
                          ),
                          Text(
                            _currentTitle,
                            style: TextStyle(fontSize: 25, color: Colors.white),
                          )
                        ],
                      ),
              ),
              Container(
                alignment: Alignment.centerRight,
                width: 30,
                child: ListView.builder(
                  physics: const NeverScrollableScrollPhysics(),
                  itemBuilder: _itemForCell,
                  itemCount: INDEX_WORDS.length,
                ),
              ),
            ],
          )),
    );
  }

  Widget _itemForCell(BuildContext context, int index) {
    return GestureDetector(
      child: Container(
        alignment: Alignment.center,
        color: _backgroundColor,
        child: Text(
          INDEX_WORDS[index],
          style: TextStyle(
            fontSize: 12,
            color: _titleColor,
          ),
        ),
      ),
    );
  }
}

int getIndex(BuildContext context, double y) {
  int index = (y ~/ (screenHeight(context) / 2 / INDEX_WORDS.length))
      .clamp(0, INDEX_WORDS.length - 1);
  print('当前的值：$index');

  return index;
}

const INDEX_WORDS = [
  '🔍',
  '☆',
  'A',
  'B',
  'C',
  'D',
  'E',
  'F',
  'G',
  'H',
  'I',
  'J',
  'K',
  'L',
  'M',
  'N',
  'O',
  'P',
  'Q',
  'R',
  'S',
  'T',
  'U',
  'V',
  'W',
  'X',
  'Y',
  'Z'
];
